//Made by JanJ123 10.3.2023
//Working with ATtiny45/85 or ATtiny25 after deleting some effects (voids), so the best option is ATtiny45, because program has around 3000 bytes of memory
//Do not forget to burn bootloader at 16MHz(internal) on ATtiny!
//ByteValues for each button on remote: CH-=69,CH=70,CH+=71,|<<=68,>>|=64,>||=67,-=7,+=21,EQ=9,0=22,100+=25,200+=13,1=12,2=24,3=94,4=8,5=28,6=90,7=66,8=82,9=74
#include <avr/io.h>
#include <util/delay.h>

#define LINE_A 1 //Pin 6 (PB1) on ATtiny85
#define LINE_B 2 //Pin 7 (PB2) on ATtiny85
#define LINE_C 3 //Pin 2 (PB3) on ATtiny85
#define LINE_D 4 //Pin 3 (PB4) on ATtiny85

int pinData= 0;   //IR receiver
unsigned long lengthHeader;
unsigned long bit;
int byteValue;

int repeat = 5; //how many times will be every effect repeated

//DDRB direction config for each LED (1 = output)
const char led_dir[10] = {
  ( 1<<LINE_B | 1<<LINE_C ), //LED 0
  ( 1<<LINE_C | 1<<LINE_B ), //LED 1
  ( 1<<LINE_A | 1<<LINE_B ), //LED 2
  ( 1<<LINE_B | 1<<LINE_A ), //LED 3
  ( 1<<LINE_D | 1<<LINE_A ), //LED 4
  
  ( 1<<LINE_A | 1<<LINE_D ), //LED 5
  ( 1<<LINE_A | 1<<LINE_C ), //LED 6
  ( 1<<LINE_C | 1<<LINE_A ), //LED 7
  ( 1<<LINE_D | 1<<LINE_B ), //LED 8
  ( 1<<LINE_B | 1<<LINE_D ), //LED 9
};

//PORTB output config for each LED (1 = High, 0 = Low)
const char led_out[10] = {
  ( 1<<LINE_B ), //LED 0
  ( 1<<LINE_C ), //LED 1
  ( 1<<LINE_A ), //LED 2
  ( 1<<LINE_B ), //LED 3
  ( 1<<LINE_D ), //LED 4
  
  ( 1<<LINE_A ), //LED 5
  ( 1<<LINE_A ), //LED 6
  ( 1<<LINE_C ), //LED 7
  ( 1<<LINE_D ), //LED 8
  ( 1<<LINE_B ), //LED 9
}; 

long ran;


void light_led(char led_num) { //led_num must be from 0 to 9
  DDRB = led_dir[led_num];
  PORTB = led_out[led_num];
}

void leds_off() {
  DDRB = 0;
  PORTB = 0;  
}

void setup(){
  pinMode(pinData, INPUT);
  for (int k=0;k<100;k++){
  light_led(0); 
  delay(1);
  light_led(1); 
  delay(1);
  light_led(2); 
  delay(1);
  light_led(3); 
  delay(1);
  light_led(4); 
  delay(1);
  light_led(5); 
  delay(1);
  light_led(6); 
  delay(1);
  light_led(7); 
  delay(1);
  light_led(8); 
  delay(1);
  light_led(9); 
  delay(1);
  }
  leds_off();
}

void random_led(){
  for (int i=0;i<40;i++){
  ran = random(0,10);
  light_led(ran);
  delay(100);
  }
}

void each_led() {
 for (int l=0;l<repeat;l++){
 for (int i=0;i<10;i++){
  light_led(i);
  delay(100);
 }
}
}

void all_blink(){
 for (int l=0;l<repeat;l++){
 for (int k=0;k<100;k++){
   for (int i=0;i<10;i++){
    light_led(i);
    delay(1);
   }
 }
 leds_off();
 delay(200);  
}
}

void alternately(){
  for (int l=0;l<repeat;l++){
  for (int k=0;k<100;k++){
   for (int i=0;i<5;i++){
    light_led(2*i);
    delay(1);
   }
 }
 for (int k=0;k<100;k++){
   for (int i=0;i<5;i++){
    light_led(2*i+1);
    delay(1);
   }
 }
 leds_off();  
}
}

void three_in_row(){
  for (int l=0;l<repeat;l++){
  for (int k=0;k<8;k++){
  for (int i=0;i<30;i++){
    light_led(k);
    delay(1);
    light_led(k+1);
    delay(1);
    light_led(k+2);
    delay(1);
 }
 }
 for (int i=0;i<30;i++){
    light_led(8);
    delay(1);
    light_led(9);
    delay(1);
    light_led(0);
    delay(1);
 }
 for (int i=0;i<30;i++){
    light_led(9);
    delay(1);
    light_led(0);
    delay(1);
    light_led(1);
    delay(1);
 }
  
}
}

void two_opposite(){
  for (int l=0;l<repeat;l++){
    for (int k=0;k<10;k++){
      for (int i=0;i<100;i++){
        light_led(k);
        delay(1);
        light_led(9-k);
        delay(1);
      }
    }
    leds_off(); 
  }
}

void blink_and_fill(){
  for (int l=0;l<repeat;l++){
    for (int k=0;k<10;k++){
      for (int j=0;j<3;j++){
        for (int i=0;i<100/(k+1);i++){
          for (int m=0;m<=k;m++){
            light_led(m);
            delay(1);
          }
        }
        for (int i=0;i<50/(k+1);i++){
          for (int m=0;m<k;m++){
            light_led(m);
            delay(1);
          }
        }
      }
    }
  }
}

void fill_one_after_moving(){
  for (int i=0;i<repeat;i++){
    for (int j=0;j<10;j++){
      for (int k=0;k<(10-j);k++){
        for (int l=0;l<100/(j+1);l++){
          for (int m=0;m<j;m++){
            light_led(9-m);
            delay(1);
          }
          light_led(k);
          delay(1);
        }
      }
    }
  }
}

void c_form(){
  for (int i=0;i<repeat;i++){
    for (int j=0;j<7;j++){
      for (int k=0;k<(10-j);k++){
        for (int l=0;l<100/(j+1);l++){
          for (int m=0;m<j;m++){
            light_led(9-m);
            delay(1);
          }
          light_led(k);
          delay(1);
        }
      }
    }
  char list[6]={7,8,9,0,1,2};
  for (int j=0;j<3;j++){
    for (int k=0;k<3;k++){
      for (int l=0;l<20;l++){
          for (int m=3-j;m<(9-j);m++){
            light_led(m);
            delay(1);
          }
          light_led(list[3-j+k]);
          delay(1);
      }
    }
      
  }
  for (int j=0;j<7;j++){
      for (int k=6-j;k<10;k++){
        for (int l=0;l<100/(7-j);l++){
          for (int m=0;m<(6-j);m++){
            light_led(m);
            delay(1);
          }
          light_led(k);
          delay(1);
        }
      }
    }
  }
}

void loop() {
  lengthHeader = pulseIn(pinData, LOW);
  if(lengthHeader > 1500){
    for(int i = 1; i <= 32; i++){
      bit = pulseIn(pinData, HIGH);
      if (i > 16 && i <= 24)
      if(bit > 1000)
      byteValue = byteValue + (1 << (i - 17)); 
    }
  }
  if(byteValue==12){  //remote 1
    each_led();
    leds_off();
  }
  if(byteValue==24){  //remote 2
    random_led();
    leds_off();
  }
  if(byteValue==94){  //remote 3
    all_blink();
    leds_off();
  }
  if(byteValue==8){   //remote 4
    alternately();
    leds_off();
  }
  if(byteValue==28){  //remote 5
    three_in_row();
    leds_off();
  }
  if(byteValue==90){  //remote 6
    two_opposite();
    leds_off();
  }
  if(byteValue==66){  //remote 7
    blink_and_fill();
    leds_off();
  }
  if(byteValue==82){  //remote 8
    fill_one_after_moving();
    leds_off();
  }
  if(byteValue==74){  //remote 9
    c_form();
    leds_off();
  }
  byteValue=0;
  delay(200);
}
